@@ -40,7 +40,10 @@ from duplicity import globals
 from duplicity import log
 from duplicity import urlparse_2_5 as urlparser
 
+from duplicity.util import exception_traceback
+
 from duplicity.errors import BackendException
+from duplicity.errors import TemporaryLoadException
 from duplicity.errors import ConflictingScheme
 from duplicity.errors import InvalidBackendURL
 from duplicity.errors import UnsupportedBackendScheme
@@ -71,12 +74,16 @@ def import_backends():
         if fn.endswith("backend.py"):
             fn = fn[:-3]
             imp = "duplicity.backends.%s" % (fn,)
+            # ignore gio as it is explicitly loaded in commandline.parse_cmdline_options()
+            if fn == "giobackend": continue
             try:
                 __import__(imp)
                 res = "Succeeded"
-            except:
+                level = log.INFO
+            except Exception:
                 res = "Failed: " + str(sys.exc_info()[1])
-            log.Info("Import of %s %s" % (imp, res))
+                level = log.WARNING
+            log.Log("Import of %s %s" % (imp, res), level)
         else:
             continue
 
@@ -150,7 +157,10 @@ def get_backend(url_string):
     elif not pu.scheme in _backends:
         raise UnsupportedBackendScheme(url_string)
     else:
-        return _backends[pu.scheme](pu)
+        try:
+            return _backends[pu.scheme](pu)
+        except ImportError:
+            raise BackendException(_("Could not initialize backend: %s") % str(sys.exc_info()[1]))
 
 
 _urlparser_initialized = False
@@ -174,11 +184,14 @@ def _ensure_urlparser_initialized():
             #
             # todo: eliminate the need for backend specific hacking here completely.
             urlparser.uses_netloc = ['ftp',
+                                     'ftps',
                                      'hsi',
                                      'rsync',
                                      's3',
-                                     'scp', 'ssh',
+                                     'u1',
+                                     'scp', 'ssh', 'sftp',
                                      'webdav', 'webdavs',
+                                     'gdocs',
                                      'http', 'https',
                                      'imap', 'imaps']
 
@@ -212,27 +225,27 @@ class ParsedUrl:
 
         try:
             pu = urlparser.urlparse(url_string)
-        except:
+        except Exception:
             raise InvalidBackendURL("Syntax error in: %s" % url_string)
 
         try:
             self.scheme = pu.scheme
-        except:
+        except Exception:
             raise InvalidBackendURL("Syntax error (scheme) in: %s" % url_string)
 
         try:
             self.netloc = pu.netloc
-        except:
+        except Exception:
             raise InvalidBackendURL("Syntax error (netloc) in: %s" % url_string)
 
         try:
             self.path = pu.path
-        except:
+        except Exception:
             raise InvalidBackendURL("Syntax error (path) in: %s" % url_string)
 
         try:
             self.username = pu.username
-        except:
+        except Exception:
             raise InvalidBackendURL("Syntax error (username) in: %s" % url_string)
         if self.username:
             self.username = urllib.unquote(pu.username)
@@ -241,7 +254,7 @@ class ParsedUrl:
 
         try:
             self.password = pu.password
-        except:
+        except Exception:
             raise InvalidBackendURL("Syntax error (password) in: %s" % url_string)
         if self.password:
             self.password = urllib.unquote(self.password)
@@ -250,14 +263,14 @@ class ParsedUrl:
 
         try:
             self.hostname = pu.hostname
-        except:
+        except Exception:
             raise InvalidBackendURL("Syntax error (hostname) in: %s" % url_string)
 
         # init to None, overwrite with actual value on success
         self.port = None
         try:
             self.port = pu.port
-        except:
+        except Exception:
             # old style rsync://host::[/]dest, are still valid, though they contain no port
             if not ( self.scheme in ['rsync'] and re.search('::[^:]*$', self.url_string)):
                 raise InvalidBackendURL("Syntax error (port) in: %s A%s B%s C%s" % (url_string, (self.scheme in ['rsync']), re.search('::[^:]+$', self.netloc), self.netloc ) )
@@ -292,6 +305,30 @@ def strip_auth_from_url(parsed_url):
     return parsed_url.geturl().replace(parsed_url.netloc, straight_netloc, 1)
 
 
+# Decorator for backend operation functions to simplify writing one that
+# retries.  Make sure to add a keyword argument 'raise_errors' to your function
+# and if it is true, raise an exception on an error.  If false, fatal-log it.
+def retry(fn):
+    def iterate(*args):
+        for n in range(1, globals.num_retries):
+            try:
+                kwargs = {"raise_errors" : True}
+                return fn(*args, **kwargs)
+            except Exception, e:
+                log.Warn("Attempt %s failed: %s: %s"
+                         % (n, e.__class__.__name__, str(e)))
+                log.Debug("Backtrace of previous error: %s"
+                          % exception_traceback())
+                if isinstance(e, TemporaryLoadException):
+                    time.sleep(30) # wait longer before trying again
+                else:
+                    time.sleep(10) # wait a bit before trying again
+        # Now try one last time, but fatal-log instead of raising errors
+        kwargs = {"raise_errors" : False}
+        return fn(*args, **kwargs)
+    return iterate
+
+
 class Backend:
     """
     Represents a generic duplicity backend, capable of storing and
@@ -304,6 +341,10 @@ class Backend:
       - list
       - delete
       - close (if needed)
+
+    Optional:
+
+      - move
     """
     def __init__(self, parsed_url):
         self.parsed_url = parsed_url
@@ -317,6 +358,16 @@ class Backend:
         """
         raise NotImplementedError()
 
+    def move(self, source_path, remote_filename = None):
+        """
+        Move source_path (Path object) to remote_filename (string)
+
+        Same as put(), but unlinks source_path in the process.  This allows the
+        local backend to do this more efficiently using rename.
+        """
+        self.put(source_path, remote_filename)
+        source_path.delete()
+
     def get(self, remote_filename, local_path):
         """Retrieve remote_filename and place in local_path"""
         raise NotImplementedError()
@@ -333,6 +384,35 @@ class Backend:
         """
         raise NotImplementedError()
 
+    # Should never cause FatalError.
+    # Returns a dictionary of dictionaries.  The outer dictionary maps
+    # filenames to metadata dictionaries.  Supported metadata are:
+    #
+    # 'size': if >= 0, size of file
+    #         if -1, file is not found
+    #         if None, error querying file
+    #
+    # Returned dictionary is guaranteed to contain a metadata dictionary for
+    # each filename, but not all metadata are guaranteed to be present.
+    def query_info(self, filename_list, raise_errors=True):
+        """
+        Return metadata about each filename in filename_list
+        """
+        info = {}
+        if hasattr(self, '_query_list_info'):
+            info = self._query_list_info(filename_list)
+        elif hasattr(self, '_query_file_info'):
+            for filename in filename_list:
+                info[filename] = self._query_file_info(filename)
+
+        # Fill out any missing entries (may happen if backend has no support
+        # or its query_list support is lazy)
+        for filename in filename_list:
+            if filename not in info:
+                info[filename] = {}
+
+        return info
+
     """ use getpass by default, inherited backends may overwrite this behaviour """
     use_getpass = True
 
@@ -364,84 +444,98 @@ class Backend:
 
         This is intended for display purposes only, and it is not
         guaranteed that the results are correct (i.e., more than just
-        the password may be substituted if the password is also a
-        substring of another part of the command line).
+        the ':password@' may be substituted.
         """
         if self.parsed_url.password:
-            return re.sub(self.parsed_url.password, '<passwd>', commandline)
+            return re.sub( r'(:([^\s:/@]+)@([^\s@]+))', r':*****@\3', commandline )
         else:
             return commandline
 
+    """
+    DEPRECATED:
+    run_command(_persist) - legacy wrappers for subprocess_popen(_persist)
+    """
     def run_command(self, commandline):
-        """
-        Execute the given command line, interpreted as a shell
-        command, with logging and error detection. If execution fails,
-        raise a BackendException.
-        """
-        private = self.munge_password(commandline)
-        log.Info(_("Running '%s'") % private)
-        if os.system(commandline):
-            raise BackendException("Error running '%s'" % private)
-
+        return self.subprocess_popen(commandline)
     def run_command_persist(self, commandline):
-        """
-        Like run_command(), but repeat the attempt several times (with
-        a delay in between) if it fails.
-        """
-        private = self.munge_password(commandline)
-        for n in range(1, globals.num_retries+1):
-            if n > 1:
-                # sleep before retry
-                time.sleep(30)
-            log.Info(gettext.ngettext("Running '%s' (attempt #%d)",
-                                      "Running '%s' (attempt #%d)", n) %
-                                      (private, n))
-            if not os.system(commandline):
-                return
-            log.Warn(gettext.ngettext("Running '%s' failed (attempt #%d)",
-                                      "Running '%s' failed (attempt #%d)", n) %
-                                      (private, n), 1)
-        log.Warn(gettext.ngettext("Giving up trying to execute '%s' after %d attempt",
-                                 "Giving up trying to execute '%s' after %d attempts",
-                                 globals.num_retries) % (private, globals.num_retries))
-        raise BackendException("Error running '%s'" % private)
+        return self.subprocess_popen_persist(commandline)
 
+    """
+    DEPRECATED:
+    popen(_persist) - legacy wrappers for subprocess_popen(_persist)
+    """
     def popen(self, commandline):
+        result, stdout, stderr = self.subprocess_popen(commandline)
+        return stdout
+    def popen_persist(self, commandline):
+        result, stdout, stderr = self.subprocess_popen_persist(commandline)
+        return stdout
+
+    def _subprocess_popen(self, commandline):
         """
-        Like run_command(), but capture stdout and return it (the
-        contents read from stdout) as a string.
+        For internal use.
+        Execute the given command line, interpreted as a shell command.
+        Returns int Exitcode, string StdOut, string StdErr
+        """
+        from subprocess import Popen, PIPE
+        p = Popen(commandline, shell=True, stdout=PIPE, stderr=PIPE)
+        stdout, stderr = p.communicate()
+
+        return p.returncode, stdout, stderr
+
+    def subprocess_popen(self, commandline):
+        """
+        Execute the given command line with error check.
+        Returns int Exitcode, string StdOut, string StdErr
+
+        Raise a BackendException on failure.
         """
         private = self.munge_password(commandline)
         log.Info(_("Reading results of '%s'") % private)
-        fout = os.popen(commandline)
-        results = fout.read()
-        if fout.close():
+        result, stdout, stderr = self._subprocess_popen(commandline)
+        if result != 0:
             raise BackendException("Error running '%s'" % private)
-        return results
+        return result, stdout, stderr
 
-    def popen_persist(self, commandline):
+    """ a dictionary for persist breaking exceptions, syntax is
+        { 'command' : [ code1, code2 ], ... } see ftpbackend for an example """
+    popen_persist_breaks = {}
+
+    def subprocess_popen_persist(self, commandline):
         """
-        Like run_command_persist(), but capture stdout and return it
-        (the contents read from stdout) as a string.
+        Execute the given command line with error check.
+        Retries globals.num_retries times with 30s delay.
+        Returns int Exitcode, string StdOut, string StdErr
+
+        Raise a BackendException on failure.
         """
         private = self.munge_password(commandline)
+
         for n in range(1, globals.num_retries+1):
+            # sleep before retry
             if n > 1:
-                # sleep before retry
                 time.sleep(30)
             log.Info(_("Reading results of '%s'") % private)
-            fout = os.popen(commandline)
-            results = fout.read()
-            result_status = fout.close()
-            if not result_status:
-                return results
-            elif result_status == 1280 and self.parsed_url.scheme == 'ftp':
-                # This squelches the "file not found" result fromm ncftpls when
-                # the ftp backend looks for a collection that does not exist.
-                return ''
-            log.Warn(gettext.ngettext("Running '%s' failed (attempt #%d)",
-                                     "Running '%s' failed (attempt #%d)", n) %
-                                      (private, n))
+            result, stdout, stderr = self._subprocess_popen(commandline)
+            if result == 0:
+                return result, stdout, stderr
+
+            try:
+                m = re.search("^\s*([\S]+)", commandline)
+                cmd = m.group(1)
+                ignores = self.popen_persist_breaks[ cmd ]
+                ignores.index(result)
+                """ ignore a predefined set of error codes """
+                return 0, '', ''
+            except (KeyError, ValueError):
+                pass
+
+            log.Warn(gettext.ngettext("Running '%s' failed with code %d (attempt #%d)",
+                                     "Running '%s' failed with code %d (attempt #%d)", n) %
+                                      (private, result, n))
+            if stdout or stderr:
+                    log.Warn(_("Error is:\n%s") % stderr + (stderr and stdout and "\n") + stdout)
+
         log.Warn(gettext.ngettext("Giving up trying to execute '%s' after %d attempt",
                                   "Giving up trying to execute '%s' after %d attempts",
                                   globals.num_retries) % (private, globals.num_retries))
