@@ -19,15 +19,13 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-import os
 import os.path
 import urllib
-import re
 
 import duplicity.backend
 from duplicity import globals
 from duplicity import log
-from duplicity.errors import *
+from duplicity.errors import * #@UnusedWildImport
 from duplicity import tempdir
 
 class FTPBackend(duplicity.backend.Backend):
@@ -40,7 +38,7 @@ class FTPBackend(duplicity.backend.Backend):
             p = os.popen("ncftpls -v")
             fout = p.read()
             ret = p.close()
-        except:
+        except Exception:
             pass
         # the expected error is 8 in the high-byte and some output
         if ret != 0x0800 or not fout:
@@ -64,6 +62,11 @@ class FTPBackend(duplicity.backend.Backend):
 
         self.url_string = duplicity.backend.strip_auth_from_url(self.parsed_url)
 
+        # This squelches the "file not found" result from ncftpls when
+        # the ftp backend looks for a collection that does not exist.
+        # version 3.2.2 has error code 5, 1280 is some legacy value
+        self.popen_persist_breaks[ 'ncftpls' ] = [ 5, 1280 ]
+
         # Use an explicit directory name.
         if self.url_string[-1] != '/':
             self.url_string += '/'
@@ -109,12 +112,12 @@ class FTPBackend(duplicity.backend.Backend):
         l = self.popen_persist(commandline).split('\n')
         l = filter(lambda x: x, l)
         # Look for our files as the last element of a long list line
-        return [x.split()[-1] for x in l]
+        return [x.split()[-1] for x in l if not x.startswith("total ")]
 
     def delete(self, filename_list):
         """Delete files in filename_list"""
         for filename in filename_list:
-            commandline = "ncftpls %s -X 'DELE %s' '%s'" % \
+            commandline = "ncftpls %s -l -X 'DELE %s' '%s'" % \
                 (self.flags, filename, self.url_string)
             self.popen_persist(commandline)
 
