@@ -27,8 +27,10 @@ import gio #@UnresolvedImport
 import glib #@UnresolvedImport
 
 import duplicity.backend
+from duplicity.backend import retry
 from duplicity import log
 from duplicity import globals
+from duplicity import util
 from duplicity.errors import * #@UnusedWildImport
 from duplicity.util import exception_traceback
 
@@ -53,12 +55,23 @@ class DupMountOperation(gio.MountOperation):
     def __init__(self, backend):
         gio.MountOperation.__init__(self)
         self.backend = backend
-        self.connect('ask-password', self.ask_password)
+        self.connect('ask-password', self.ask_password_cb)
+        self.connect('ask-question', self.ask_question_cb)
 
-    def ask_password(self, *args, **kwargs):
+    def ask_password_cb(self, *args, **kwargs):
         self.set_password(self.backend.get_password())
         self.reply(gio.MOUNT_OPERATION_HANDLED)
 
+    def ask_question_cb(self, *args, **kwargs):
+        # Obviously just always answering with the first choice is a naive
+        # approach.  But there's no easy way to allow for answering questions
+        # in duplicity's typical run-from-cron mode with environment variables.
+        # And only a couple gvfs backends ask questions: 'sftp' does about
+        # new hosts and 'afc' does if the device is locked.  0 should be a
+        # safe choice.
+        self.set_choice(0)
+        self.reply(gio.MOUNT_OPERATION_HANDLED)
+
 class GIOBackend(duplicity.backend.Backend):
     """Use this backend when saving to a GIO URL.
        This is a bit of a meta-backend, in that it can handle multiple schemas.
@@ -88,58 +101,94 @@ class GIOBackend(duplicity.backend.Backend):
                                % str(e), log.ErrorCode.connection_failed)
         loop.quit()
 
+    def handle_error(self, raise_error, e, op, file1=None, file2=None):
+        if raise_error:
+            raise e
+        code = log.ErrorCode.backend_error
+        if isinstance(e, gio.Error):
+            if e.code == gio.ERROR_PERMISSION_DENIED:
+                code = log.ErrorCode.backend_permission_denied
+            elif e.code == gio.ERROR_NOT_FOUND:
+                code = log.ErrorCode.backend_not_found
+            elif e.code == gio.ERROR_NO_SPACE:
+                code = log.ErrorCode.backend_no_space
+        extra = ' '.join([util.escape(x) for x in [file1, file2] if x])
+        extra = ' '.join([op, extra])
+        log.FatalError(str(e), code, extra)
+
     def copy_progress(self, *args, **kwargs):
         pass
 
-    def copy_file(self, source, target):
-        for n in range(1, globals.num_retries+1):
-            log.Info(_("Writing %s") % target.get_parse_name())
-            try:
-                source.copy(target, self.copy_progress,
-                            gio.FILE_COPY_OVERWRITE | gio.FILE_COPY_NOFOLLOW_SYMLINKS)
-                return
-            except Exception, e:
-                log.Warn("Write of '%s' failed (attempt %s): %s: %s"
-                        % (target.get_parse_name(), n, e.__class__.__name__, str(e)))
-                log.Debug("Backtrace of previous error: %s"
-                          % exception_traceback())
-        raise BackendException(_("Could not copy %s to %s") % (source.get_parse_name(),
-                                                               target.get_parse_name()))
+    @retry
+    def copy_file(self, op, source, target, raise_errors=False):
+        log.Info(_("Writing %s") % target.get_parse_name())
+        try:
+            source.copy(target, self.copy_progress,
+                        gio.FILE_COPY_OVERWRITE | gio.FILE_COPY_NOFOLLOW_SYMLINKS)
+        except Exception, e:
+            self.handle_error(raise_errors, e, op, source.get_parse_name(),
+                              target.get_parse_name())
 
     def put(self, source_path, remote_filename = None):
         """Copy file to remote"""
         if not remote_filename:
             remote_filename = source_path.get_filename()
         source_file = gio.File(path=source_path.name)
-        target_file = self.remote_file.get_child_for_display_name(remote_filename)
-        self.copy_file(source_file, target_file)
+        target_file = self.remote_file.get_child(remote_filename)
+        self.copy_file('put', source_file, target_file)
 
     def get(self, filename, local_path):
         """Get file and put in local_path (Path object)"""
-        source_file = self.remote_file.get_child_for_display_name(filename)
+        source_file = self.remote_file.get_child(filename)
         target_file = gio.File(path=local_path.name)
-        self.copy_file(source_file, target_file)
+        self.copy_file('get', source_file, target_file)
         local_path.setdata()
 
-    def list(self):
+    @retry
+    def list(self, raise_errors=False):
         """List files in that directory"""
-        enum = self.remote_file.enumerate_children(gio.FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME,
-                                                   gio.FILE_QUERY_INFO_NOFOLLOW_SYMLINKS)
         files = []
         try:
+            enum = self.remote_file.enumerate_children(gio.FILE_ATTRIBUTE_STANDARD_NAME,
+                                                       gio.FILE_QUERY_INFO_NOFOLLOW_SYMLINKS)
             info = enum.next_file()
             while info:
-                files.append(info.get_display_name())
+                files.append(info.get_name())
                 info = enum.next_file()
-            return files
         except Exception, e:
-            raise BackendException(str(e))
+            self.handle_error(raise_errors, e, 'list',
+                              self.remote_file.get_parse_name())
+        return files
 
-    def delete(self, filename_list):
+    @retry
+    def delete(self, filename_list, raise_errors=False):
         """Delete all files in filename list"""
         assert type(filename_list) is not types.StringType
+        for filename in filename_list:
+            target_file = self.remote_file.get_child(filename)
+            try:
+                target_file.delete()
+            except Exception, e:
+                if isinstance(e, gio.Error):
+                    if e.code == gio.ERROR_NOT_FOUND:
+                        continue
+                self.handle_error(raise_errors, e, 'delete',
+                                  target_file.get_parse_name())
+                return
+
+    @retry
+    def _query_file_info(self, filename, raise_errors=False):
+        """Query attributes on filename"""
+        target_file = self.remote_file.get_child(filename)
+        attrs = gio.FILE_ATTRIBUTE_STANDARD_SIZE
         try:
-                for filename in filename_list:
-                        self.remote_file.get_child_for_display_name(filename).delete()
+            info = target_file.query_info(attrs, gio.FILE_QUERY_INFO_NONE)
+            return {'size': info.get_size()}
         except Exception, e:
-            raise BackendException(str(e))
+            if isinstance(e, gio.Error):
+                if e.code == gio.ERROR_NOT_FOUND:
+                    return {'size': -1} # early exit, no need to retry
+            if raise_errors:
+                raise e
+            else:
+                return {'size': None}
