@@ -19,7 +19,6 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-import os
 import base64
 import httplib
 import re
@@ -30,7 +29,7 @@ import xml.dom.minidom
 import duplicity.backend
 from duplicity import globals
 from duplicity import log
-from duplicity.errors import *
+from duplicity.errors import * #@UnusedWildImport
 from duplicity import urlparse_2_5 as urlparser
 
 class CustomMethodRequest(urllib2.Request):
@@ -52,13 +51,15 @@ class WebDAVBackend(duplicity.backend.Backend):
 
     webdav backend contributed in 2006 by Jesper Zedlitz <jesper@zedlitz.de>
     """
-    listbody = """\
-<?xml version="1.0" encoding="utf-8" ?>
-<D:propfind xmlns:D="DAV:">
-<D:allprop/>
-</D:propfind>
 
-"""
+    """
+    for better compatibility we send an empty listbody as described in
+    http://www.ietf.org/rfc/rfc4918.txt
+    "  A client may choose not to submit a request body.  An empty PROPFIND
+       request body MUST be treated as if it were an 'allprop' request.  "
+    it was retired because e.g. box.net didn't support <D:allprop/>
+    """
+    listbody =""
 
     """Connect to remote store using WebDAV Protocol"""
     def __init__(self, parsed_url):
@@ -75,13 +76,14 @@ class WebDAVBackend(duplicity.backend.Backend):
             self.directory = '/'
 
         log.Info("Using WebDAV host %s" % (parsed_url.hostname,))
+        log.Info("Using WebDAV port %s" % (parsed_url.port,))
         log.Info("Using WebDAV directory %s" % (self.directory,))
         log.Info("Using WebDAV protocol %s" % (globals.webdav_proto,))
 
         if parsed_url.scheme == 'webdav':
-            self.conn = httplib.HTTPConnection(parsed_url.hostname)
+            self.conn = httplib.HTTPConnection(parsed_url.hostname, parsed_url.port)
         elif parsed_url.scheme == 'webdavs':
-            self.conn = httplib.HTTPSConnection(parsed_url.hostname)
+            self.conn = httplib.HTTPSConnection(parsed_url.hostname, parsed_url.port)
         else:
             raise BackendException("Unknown URI scheme: %s" % (parsed_url.scheme))
 
@@ -107,6 +109,7 @@ class WebDAVBackend(duplicity.backend.Backend):
         self.conn.request(method, quoted_path, data, self.headers)
         response = self.conn.getresponse()
         if response.status == 401:
+            response.close()
             self.headers['Authorization'] = self.get_authorization(response, quoted_path)
             self.conn.request(method, quoted_path, data, self.headers)
             response = self.conn.getresponse()
@@ -166,12 +169,15 @@ class WebDAVBackend(duplicity.backend.Backend):
             del self.headers['Depth']
             # if the target collection does not exist, create it.
             if response.status == 404:
+                response.close()
                 log.Info("Directory '%s' being created." % self.directory)
-                res = self.request("MKCOL", self.directory)
-                log.Info("WebDAV MKCOL status: %s %s" % (res.status, res.reason))
+                response = self.request("MKCOL", self.directory)
+                log.Info("WebDAV MKCOL status: %s %s" % (response.status, response.reason))
+                response.close()
                 continue
             if response.status == 207:
                 document = response.read()
+                response.close()
                 break
             log.Info("WebDAV PROPFIND attempt #%d failed: %s %s" % (n, response.status, response.reason))
             if n == globals.num_retries +1:
@@ -181,7 +187,7 @@ class WebDAVBackend(duplicity.backend.Backend):
         log.Info("%s" % (document,))
         dom = xml.dom.minidom.parseString(document)
         result = []
-        for href in dom.getElementsByTagName('D:href'):
+        for href in dom.getElementsByTagName('d:href') + dom.getElementsByTagName('D:href'):
             filename = self.__taste_href(href)
             if filename:
                 result.append(filename)
