@@ -141,8 +141,9 @@ def get_index_from_tarinfo( tarinfo ):
     """Return (index, difftype, multivol) pair from tarinfo object"""
     for prefix in ["snapshot/", "diff/", "deleted/",
                    "multivol_diff/", "multivol_snapshot/"]:
-        if tarinfo.name.startswith( prefix ):
-            name = tarinfo.name[len( prefix ):] # strip prefix
+        tiname = util.get_tarinfo_name( tarinfo )
+        if tiname.startswith( prefix ):
+            name = tiname[len( prefix ):] # strip prefix
             if prefix.startswith( "multivol" ):
                 if prefix == "multivol_diff/":
                     difftype = "diff"
@@ -150,28 +151,28 @@ def get_index_from_tarinfo( tarinfo ):
                     difftype = "snapshot"
                 multivol = 1
                 name, num_subs = \
-                      re.subn( "(?s)^multivol_(diff|snapshot)/(.*)/[0-9]+$",
-                              "\\2", tarinfo.name )
+                      re.subn( "(?s)^multivol_(diff|snapshot)/?(.*)/[0-9]+$",
+                              "\\2", tiname )
                 if num_subs != 1:
                     raise PatchDirException( "Unrecognized diff entry %s" %
-                                            ( tarinfo.name, ) )
+                                            ( tiname, ) )
             else:
                 difftype = prefix[:-1] # strip trailing /
-                name = tarinfo.name[len( prefix ):]
+                name = tiname[len( prefix ):]
                 if name.endswith( "/" ):
                     name = name[:-1] # strip trailing /'s
                 multivol = 0
             break
     else:
         raise PatchDirException( "Unrecognized diff entry %s" %
-                                 ( tarinfo.name, ) )
+                                 ( tiname, ) )
     if name == "." or name == "":
         index = ()
     else:
         index = tuple( name.split( "/" ) )
         if '..' in index:
             raise PatchDirException( "Tar entry %s contains '..'.  Security "
-                                    "violation" % ( tarinfo.name, ) )
+                                    "violation" % ( tiname, ) )
     return ( index, difftype, multivol )
 
 
@@ -320,7 +321,7 @@ class TarFile_FromFileobjs:
         if self.current_fp:
             assert not self.current_fp.close()
         self.current_fp = self.fileobj_iter.next()
-        self.tarfile = tarfile.TarFile( "arbitrary", "r", self.current_fp )
+        self.tarfile = util.make_tarfile("r", self.current_fp)
         self.tar_iter = iter( self.tarfile )
 
     def next( self ):
